@@ -18,7 +18,7 @@ require Exporter;
 use vars qw(@ISA @EXPORT_OK);
 
 @ISA = qw(Exporter);
-@EXPORT_OK = qw(access_data_cache);
+@EXPORT_OK = qw(access_data_cache cgi_request_args);
 
 sub access_data_cache
 {
@@ -262,7 +262,7 @@ sub access_data_cache
 	$expired = 1 if ($mem->{expires} && $time >= $mem->{expires});
 	if (exists($options{expire_if})) {
 	    my $sub = $options{expire_if};
-	    $expired = 1 if (&$sub($mem->{lastModified}));
+	    $expired = 1 if (&$sub($mem->{lastModified} || 0));
 	}
 	return $mem->{contents} if !$expired;
 	return undef if !$options{busy_lock};
@@ -310,6 +310,35 @@ sub access_data_cache
     }
 }
 
+#
+# Given an initialized CGI object, return a hash of arguments for
+# passing to a top-level component. Used by both ApacheHandler.pm
+# and pure CGI Mason implementations.
+#
+sub cgi_request_args
+{
+    my ($q, $method) = @_;
+
+    my %args;
+    my $param_methods = $method eq 'GET' ? [ 'param' ] : [ 'param', 'url_param' ];
+    foreach my $method (@$param_methods) {
+	foreach my $key ( $q->$method() ) {
+	    foreach my $value ( $q->$method($key) ) {
+		if (exists($args{$key})) {
+		    if (ref($args{$key}) eq 'ARRAY') {
+			push @{ $args{$key} }, $value;
+		    } else {
+			$args{$key} = [$args{$key}, $value];
+		    }
+		} else {
+		    $args{$key} = $value;
+		}
+	    }
+	}
+    }
+    return %args;
+}
+
 #
 # Returns 1 if the exclusive, non-blocking lock was obtained,
 # undef otherwise. Left here for content management
